/*!
FullCalendar v5.11.0
Docs & License: https://fullcalendar.io/
(c) 2021 Adam Shaw
*/
var FullCalendarCopyPaste = (function (exports, common) {
    'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    /*
    Is a cache for a given element's scroll information (all the info that ScrollController stores)
    in addition the "client rectangle" of the element.. the area within the scrollbars.

    The cache can be in one of two modes:
    - doesListening:false - ignores when the container is scrolled by someone else
    - doesListening:true - watch for scrolling and update the cache
    */
    var ScrollGeomCache = /** @class */ (function (_super) {
        __extends(ScrollGeomCache, _super);
        function ScrollGeomCache(scrollController, doesListening) {
            var _this = _super.call(this) || this;
            _this.handleScroll = function () {
                _this.scrollTop = _this.scrollController.getScrollTop();
                _this.scrollLeft = _this.scrollController.getScrollLeft();
                _this.handleScrollChange();
            };
            _this.scrollController = scrollController;
            _this.doesListening = doesListening;
            _this.scrollTop = _this.origScrollTop = scrollController.getScrollTop();
            _this.scrollLeft = _this.origScrollLeft = scrollController.getScrollLeft();
            _this.scrollWidth = scrollController.getScrollWidth();
            _this.scrollHeight = scrollController.getScrollHeight();
            _this.clientWidth = scrollController.getClientWidth();
            _this.clientHeight = scrollController.getClientHeight();
            _this.clientRect = _this.computeClientRect(); // do last in case it needs cached values
            if (_this.doesListening) {
                _this.getEventTarget().addEventListener('scroll', _this.handleScroll);
            }
            return _this;
        }
        ScrollGeomCache.prototype.destroy = function () {
            if (this.doesListening) {
                this.getEventTarget().removeEventListener('scroll', this.handleScroll);
            }
        };
        ScrollGeomCache.prototype.getScrollTop = function () {
            return this.scrollTop;
        };
        ScrollGeomCache.prototype.getScrollLeft = function () {
            return this.scrollLeft;
        };
        ScrollGeomCache.prototype.setScrollTop = function (top) {
            this.scrollController.setScrollTop(top);
            if (!this.doesListening) {
                // we are not relying on the element to normalize out-of-bounds scroll values
                // so we need to sanitize ourselves
                this.scrollTop = Math.max(Math.min(top, this.getMaxScrollTop()), 0);
                this.handleScrollChange();
            }
        };
        ScrollGeomCache.prototype.setScrollLeft = function (top) {
            this.scrollController.setScrollLeft(top);
            if (!this.doesListening) {
                // we are not relying on the element to normalize out-of-bounds scroll values
                // so we need to sanitize ourselves
                this.scrollLeft = Math.max(Math.min(top, this.getMaxScrollLeft()), 0);
                this.handleScrollChange();
            }
        };
        ScrollGeomCache.prototype.getClientWidth = function () {
            return this.clientWidth;
        };
        ScrollGeomCache.prototype.getClientHeight = function () {
            return this.clientHeight;
        };
        ScrollGeomCache.prototype.getScrollWidth = function () {
            return this.scrollWidth;
        };
        ScrollGeomCache.prototype.getScrollHeight = function () {
            return this.scrollHeight;
        };
        ScrollGeomCache.prototype.handleScrollChange = function () {
        };
        return ScrollGeomCache;
    }(common.ScrollController));

    var ElementScrollGeomCache = /** @class */ (function (_super) {
        __extends(ElementScrollGeomCache, _super);
        function ElementScrollGeomCache(el, doesListening) {
            return _super.call(this, new common.ElementScrollController(el), doesListening) || this;
        }
        ElementScrollGeomCache.prototype.getEventTarget = function () {
            return this.scrollController.el;
        };
        ElementScrollGeomCache.prototype.computeClientRect = function () {
            return common.computeInnerRect(this.scrollController.el);
        };
        return ElementScrollGeomCache;
    }(ScrollGeomCache));

    /*
    When this class is instantiated, it records the offset of an element (relative to the document topleft),
    and continues to monitor scrolling, updating the cached coordinates if it needs to.
    Does not access the DOM after instantiation, so highly performant.

    Also keeps track of all scrolling/overflow:hidden containers that are parents of the given element
    and an determine if a given point is inside the combined clipping rectangle.
    */
    var OffsetTracker = /** @class */ (function () {
        function OffsetTracker(el) {
            this.origRect = common.computeRect(el);
            // will work fine for divs that have overflow:hidden
            this.scrollCaches = common.getClippingParents(el).map(function (scrollEl) { return new ElementScrollGeomCache(scrollEl, true); });
        }
        OffsetTracker.prototype.destroy = function () {
            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {
                var scrollCache = _a[_i];
                scrollCache.destroy();
            }
        };
        OffsetTracker.prototype.computeLeft = function () {
            var left = this.origRect.left;
            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {
                var scrollCache = _a[_i];
                left += scrollCache.origScrollLeft - scrollCache.getScrollLeft();
            }
            return left;
        };
        OffsetTracker.prototype.computeTop = function () {
            var top = this.origRect.top;
            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {
                var scrollCache = _a[_i];
                top += scrollCache.origScrollTop - scrollCache.getScrollTop();
            }
            return top;
        };
        OffsetTracker.prototype.isWithinClipping = function (pageX, pageY) {
            var point = { left: pageX, top: pageY };
            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {
                var scrollCache = _a[_i];
                if (!isIgnoredClipping(scrollCache.getEventTarget()) &&
                    !common.pointInsideRect(point, scrollCache.clientRect)) {
                    return false;
                }
            }
            return true;
        };
        return OffsetTracker;
    }());
    // certain clipping containers should never constrain interactions, like <html> and <body>
    // https://github.com/fullcalendar/fullcalendar/issues/3615
    function isIgnoredClipping(node) {
        var tagName = node.tagName;
        return tagName === 'HTML' || tagName === 'BODY';
    }

    var HitChecker = /** @class */ (function () {
        function HitChecker(dragging, droppableStore) {
            var _this = this;
            this.type = null;
            // options that can be set by caller
            this.useSubjectCenter = false;
            this.requireInitial = true; // if doesn't start out on a hit, won't emit any events
            this.initialHit = null;
            this.finalHit = null;
            this.movingHit = null;
            this.checkValidBehavior = function () {
                return _this.type !== null;
            };
            this.handleCopy = function (ev) {
                _this.type = 'copy';
                _this.prevHandle(ev);
                _this.emitter.trigger('pointer-copy', ev);
            };
            this.handleCut = function (ev) {
                _this.type = 'cut';
                _this.prevHandle(ev);
                _this.emitter.trigger('pointer-cut', ev);
            };
            this.handleDuplicate = function (ev) {
                _this.type = 'duplicate';
                _this.prevHandle(ev);
                _this.emitter.trigger('pointer-duplicate', ev);
            };
            this.handleMove = function (ev, forceHandle) {
                if (!_this.checkValidBehavior())
                    return;
                var hit = _this.queryHitForOffset(ev.pageX + _this.coordAdjust.left, ev.pageY + _this.coordAdjust.top, 
                //@ts-ignore
                ev.isFromGlobal);
                if (forceHandle || !isHitsEqual(_this.movingHit, hit)) {
                    _this.movingHit = hit;
                    _this.emitter.trigger('hitupdate', hit, false, ev);
                }
            };
            this.handlePaste = function (ev) {
                if (!_this.initialHit)
                    return;
                _this.finalHit = _this.queryHitForOffset(ev.pageX + _this.coordAdjust.left, ev.pageY + _this.coordAdjust.top, 
                //@ts-ignore
                ev.isFromGlobal);
                if (!_this.finalHit)
                    return;
                _this.emitter.trigger('paste', ev);
            };
            this.prevHandle = function (ev) {
                _this.initialHit = null;
                _this.finalHit = null;
                _this.prepareHits();
                _this.processFirstCoord(ev);
                _this.handleMove(ev, true);
            };
            this.cleanup = function () {
                _this.initialHit = null;
                _this.finalHit = null;
                _this.type = null;
                _this.emitter.trigger('cleanup', true);
            };
            this.droppableStore = droppableStore;
            dragging.emitter.on('pointer-copy', this.handleCopy);
            dragging.emitter.on('pointer-cut', this.handleCut);
            dragging.emitter.on('pointer-duplicate', this.handleDuplicate);
            dragging.emitter.on('pointer-paste', this.handlePaste);
            dragging.emitter.on('mousemove', this.handleMove);
            dragging.emitter.on('cleanup', this.cleanup);
            // @ts-ignore
            this.dragging = dragging;
            this.emitter = new common.Emitter();
        }
        HitChecker.prototype.processFirstCoord = function (ev) {
            var origPoint = { left: ev.pageX, top: ev.pageY };
            var adjustedPoint = origPoint;
            var subjectEl = ev.subjectEl;
            var subjectRect;
            if (subjectEl instanceof HTMLElement) {
                subjectRect = common.computeRect(subjectEl);
                adjustedPoint = common.constrainPoint(adjustedPoint, subjectRect);
            }
            //@ts-ignore
            var initialHit = this.initialHit = this.queryHitForOffset(adjustedPoint.left, adjustedPoint.top, ev.isFromGlobal);
            if (initialHit !== null) {
                if (this.useSubjectCenter && subjectRect) {
                    var slicedSubjectRect = common.intersectRects(subjectRect, initialHit.rect);
                    if (slicedSubjectRect) {
                        adjustedPoint = common.getRectCenter(slicedSubjectRect);
                    }
                }
                this.coordAdjust = common.diffPoints(adjustedPoint, origPoint);
            }
            else {
                this.coordAdjust = { left: 0, top: 0 };
            }
        };
        HitChecker.prototype.prepareHits = function () {
            this.offsetTrackers = common.mapHash(this.droppableStore, function (interactionSettings) {
                interactionSettings.component.prepareHits();
                return new OffsetTracker(interactionSettings.el);
            });
        };
        HitChecker.prototype.releaseHits = function () {
            var offsetTrackers = this.offsetTrackers;
            for (var id in offsetTrackers) {
                offsetTrackers[id].destroy();
            }
            this.offsetTrackers = {};
        };
        HitChecker.prototype.queryHitForOffset = function (offsetLeft, offsetTop, isFromGlobal) {
            if (isFromGlobal === void 0) { isFromGlobal = false; }
            var _a = this, droppableStore = _a.droppableStore, offsetTrackers = _a.offsetTrackers;
            var bestHit = null;
            for (var id in droppableStore) {
                var component = droppableStore[id].component;
                var offsetTracker = offsetTrackers[id];
                if (offsetTracker && (isFromGlobal || !isFromGlobal && offsetTracker.isWithinClipping(offsetLeft, offsetTop))) {
                    var originLeft = offsetTracker.computeLeft();
                    var originTop = offsetTracker.computeTop();
                    var positionLeft = offsetLeft - originLeft;
                    var positionTop = offsetTop - originTop;
                    var origRect = offsetTracker.origRect;
                    var width = origRect.right - origRect.left;
                    var height = origRect.bottom - origRect.top;
                    if (
                    // must be within the element's bounds
                    positionLeft >= 0 && positionLeft < width &&
                        positionTop >= 0 && positionTop < height) {
                        var hit = component.queryHit(positionLeft, positionTop, width, height);
                        if (hit && (
                        // make sure the hit is within activeRange, meaning it's not a dead cell
                        common.rangeContainsRange(hit.dateProfile.activeRange, hit.dateSpan.range)) &&
                            (!bestHit || hit.layer > bestHit.layer)) {
                            hit.componentId = id;
                            hit.context = component.context;
                            // TODO: better way to re-orient rectangle
                            hit.rect.left += originLeft;
                            hit.rect.right += originLeft;
                            hit.rect.top += originTop;
                            hit.rect.bottom += originTop;
                            bestHit = hit;
                        }
                    }
                }
            }
            return bestHit;
        };
        return HitChecker;
    }());
    function isHitsEqual(hit0, hit1) {
        if (!hit0 && !hit1) {
            return true;
        }
        if (Boolean(hit0) !== Boolean(hit1)) {
            return false;
        }
        return common.isDateSpansEqual(hit0.dateSpan, hit1.dateSpan);
    }

    //@ts-ignore
    window.calendarUtils = window.calendarUtils || {};
    //@ts-ignore
    window.calendarUtils.emitter = window.calendarUtils.emitter || new common.Emitter();
    function copy(event) {
        //@ts-ignore
        window.calendarUtils.emitter.trigger('copy', event);
    }
    function cut(event) {
        //@ts-ignore
        window.calendarUtils.emitter.trigger('cut', event);
    }
    //@ts-ignore
    window.calendarUtils.copy = window.calendarUtils.copy || copy;
    //@ts-ignore
    window.calendarUtils.cut = window.calendarUtils.cut || cut;
    var KEY_META = 'Meta';
    var KEY_C = 'c';
    var KEY_V = 'v';
    var KEY_D = 'd';
    var KEY_X = 'x';
    var KEY_ESCAPE = 'Escape';
    var KEY_ENTER = 'Enter';
    var allowKeyboard = [KEY_META, KEY_C, KEY_V, KEY_D, KEY_X];
    var CONTAINER_CLASS = '.fc-timeline-body, .fc-timegrid-body, .fc-daygrid-body';
    var PointerTracking = /** @class */ (function () {
        function PointerTracking(containerEl) {
            var _this = this;
            this.subjectEl = null;
            this.pressedMetaKey = false;
            this.listKey = {};
            this.type = TYPE_EVENT.NONE;
            this.lastPoint = null;
            this.isMac = navigator.userAgent.includes('Mac');
            // options that can be directly assigned by caller
            this.selector = ''; // will cause subjectEl in all emitted events to be this element
            this.handleSelector = '';
            this.tryStart = function (ev) {
                var subjectEl = _this.querySubjectEl(ev);
                var fcTimeEl = ev.target.closest(CONTAINER_CLASS);
                var containerFcTimeEl = _this.containerEl.closest(CONTAINER_CLASS);
                if (subjectEl && fcTimeEl === containerFcTimeEl) {
                    _this.subjectEl = subjectEl;
                    return true;
                }
                return false;
            };
            this.globalTryStart = function (element) {
                var subjectEl = _this.queryEl(element);
                var fcTimeEl = element.closest(CONTAINER_CLASS);
                var containerFcTimeEl = _this.containerEl.closest(CONTAINER_CLASS);
                if (subjectEl && fcTimeEl === containerFcTimeEl) {
                    _this.subjectEl = subjectEl;
                    return true;
                }
                return false;
            };
            this.cleanup = function () {
                _this.type = TYPE_EVENT.NONE;
                _this.subjectEl = null;
                _this.listKey = {};
                _this.emitter.trigger('cleanup', true);
            };
            this.handleMouseDown = function (event) {
                if (_this.lastPoint && _this.type !== TYPE_EVENT.NONE) {
                    _this.emitter.trigger('mousedown', _this.createEventFromMouse(_this.lastPoint));
                }
                setTimeout(function () { return _this.cleanup(); }, 50);
            };
            // Keyboard
            // ----------------------------------------------------------------------------------------------------
            this.handleKeyDown = function (ev) {
                if (_this.listKey[ev.key] !== void 0)
                    return;
                if (allowKeyboard.includes(ev.key)) {
                    _this.listKey[ev.key] = ev;
                }
                _this.checkPrimaryKey();
                _this.handleCopyPaste(ev);
            };
            this.handleKeyUp = function (ev) {
                delete _this.listKey[ev.key];
                if (ev.key === KEY_META) {
                    _this.listKey = {};
                }
            };
            this.checkPrimaryKey = function () {
                _this.pressedMetaKey = false;
                Object.values(_this.listKey).forEach(function (value) {
                    if (value.key === KEY_META) {
                        _this.pressedMetaKey = true;
                    }
                });
            };
            this.handleCopyPaste = function (event) {
                // for MacOS or Window keyboard
                if (_this.lastPoint) {
                    if (event.key === KEY_ENTER) {
                        return _this.handlePaste();
                    }
                    else if (event.key === KEY_ESCAPE) {
                        return _this.cleanup();
                    }
                    if ((_this.isMac && _this.pressedMetaKey || !_this.isMac && event.ctrlKey)) {
                        if (event.key === KEY_C) {
                            _this.handleCopy();
                        }
                        else if (event.key === KEY_V) {
                            _this.handlePaste();
                        }
                        else if (event.key === KEY_X) {
                            _this.handleCut();
                        }
                        else if (event.key === KEY_D) {
                            _this.handleDuplicate(event);
                        }
                    }
                }
            };
            this.handleCopy = function () {
                if (_this.tryStart(_this.lastPoint)) {
                    _this.type = TYPE_EVENT.COPY;
                    var pev = _this.createEventFromMouse(_this.lastPoint, true);
                    _this.emitter.trigger('pointer-copy', pev);
                }
                else {
                    _this.cleanup();
                }
            };
            this.handlePaste = function () {
                if (_this.type === TYPE_EVENT.NONE)
                    return;
                _this.emitter.trigger('pointer-paste', _this.createEventFromMouse(_this.lastPoint));
                setTimeout(function () { return _this.cleanup(); }, 50);
            };
            this.handleCut = function () {
                if (_this.tryStart(_this.lastPoint)) {
                    _this.type = TYPE_EVENT.CUT;
                    var pev = _this.createEventFromMouse(_this.lastPoint, true);
                    _this.emitter.trigger('pointer-cut', pev);
                }
                else {
                    _this.cleanup();
                }
            };
            this.handleDuplicate = function (event) {
                _this.handleCopy();
                if (_this.type === TYPE_EVENT.NONE)
                    return;
                event.preventDefault();
                _this.handlePaste();
            };
            this.handleMouseMove = function (ev) {
                _this.lastPoint = ev;
                _this.emitter.trigger('mousemove', _this.createEventFromMouse(ev, true));
            };
            // Event from global
            // ----------------------------------------------------------------------------------------------------
            this.handleGlobalCopy = function (el) {
                if (el && _this.globalTryStart(el)) {
                    _this.type = TYPE_EVENT.COPY;
                    var pev = _this.createEventFromElement(_this.subjectEl);
                    _this.emitter.trigger('pointer-copy', pev);
                }
                else {
                    _this.cleanup();
                }
            };
            this.handleGlobalCut = function (el) {
                if (el && _this.globalTryStart(el)) {
                    _this.type = TYPE_EVENT.CUT;
                    var pev = _this.createEventFromElement(_this.subjectEl);
                    _this.emitter.trigger('pointer-cut', pev);
                }
                else {
                    _this.cleanup();
                }
            };
            this.containerEl = containerEl;
            this.emitter = new common.Emitter();
            document.body.addEventListener('mousedown', this.handleMouseDown, false);
            document.body.addEventListener('mousemove', this.handleMouseMove, false);
            document.body.addEventListener('keydown', this.handleKeyDown, false);
            document.body.addEventListener('keyup', this.handleKeyUp, false);
            // global utils
            // @ts-ignore
            window.calendarUtils.emitter.on('copy', this.handleGlobalCopy);
            // @ts-ignore
            window.calendarUtils.emitter.on('cut', this.handleGlobalCut);
        }
        PointerTracking.prototype.destroy = function () {
            document.body.removeEventListener('mousedown', this.handleMouseDown, false);
            document.body.removeEventListener('mousemove', this.handleMouseMove, false);
            document.body.removeEventListener('keydown', this.handleKeyDown, false);
            document.body.removeEventListener('keyup', this.handleKeyUp, false);
            // @ts-ignore
            window.calendarUtils.emitter.off('copy', this.handleGlobalCopy);
            // @ts-ignore
            window.calendarUtils.emitter.off('cut', this.handleGlobalCut);
        };
        PointerTracking.prototype.querySubjectEl = function (ev) {
            if (this.selector) {
                return common.elementClosest(ev.target, this.selector);
            }
            return this.containerEl;
        };
        PointerTracking.prototype.queryEl = function (el) {
            if (this.selector) {
                return el.closest(this.selector);
            }
            return this.containerEl;
        };
        // Event Normalization
        // ----------------------------------------------------------------------------------------------------
        PointerTracking.prototype.createEventFromMouse = function (ev, isFirst) {
            var deltaX = 0;
            var deltaY = 0;
            // TODO: repeat code
            if (isFirst) {
                this.origPageX = ev.pageX;
                this.origPageY = ev.pageY;
            }
            else {
                deltaX = ev.pageX - this.origPageX;
                deltaY = ev.pageY - this.origPageY;
            }
            return {
                origEvent: ev,
                isTouch: false,
                subjectEl: this.subjectEl,
                pageX: ev.pageX,
                pageY: ev.pageY,
                deltaX: deltaX,
                deltaY: deltaY
            };
        };
        PointerTracking.prototype.createEventFromElement = function (el) {
            var rect = el.getBoundingClientRect();
            return {
                // @ts-ignore
                origEvent: { target: el },
                isFromGlobal: true,
                subjectEl: this.subjectEl,
                pageX: rect.x + (rect.width / 2) + window.scrollX,
                pageY: rect.y + (rect.height / 2) + window.scrollY,
                deltaX: 0,
                deltaY: 0
            };
        };
        return PointerTracking;
    }());

    /*
    An effect in which an element follows the movement of a pointer across the screen.
    The moving element is a clone of some other element.
    Must call start + handleMove + stop.
    */
    var ElementMirror = /** @class */ (function () {
        function ElementMirror() {
            this.isVisible = false; // must be explicitly enabled
            this.sourceEl = null;
            this.mirrorEl = null;
            this.sourceElRect = null; // screen coords relative to viewport
            // options that can be set directly by caller
            this.parentNode = document.body; // HIGHLY SUGGESTED to set this to sidestep ShadowDOM issues
            this.zIndex = 9999;
            this.revertDuration = 0;
        }
        ElementMirror.prototype.start = function (sourceEl, pageX, pageY) {
            this.sourceEl = sourceEl;
            this.sourceElRect = this.sourceEl.getBoundingClientRect();
            this.origScreenX = pageX - window.scrollX;
            this.origScreenY = pageY - window.scrollY;
            this.deltaX = 0;
            this.deltaY = 0;
            this.updateElPosition();
        };
        ElementMirror.prototype.handleMove = function (pageX, pageY) {
            this.deltaX = (pageX - window.scrollX) - this.origScreenX;
            this.deltaY = (pageY - window.scrollY) - this.origScreenY;
            this.updateElPosition();
        };
        // can be called before start
        ElementMirror.prototype.setIsVisible = function (bool) {
            if (bool) {
                if (!this.isVisible) {
                    if (this.mirrorEl) {
                        this.mirrorEl.style.display = '';
                    }
                    this.isVisible = bool; // needs to happen before updateElPosition
                    this.updateElPosition(); // because was not updating the position while invisible
                }
            }
            else if (this.isVisible) {
                if (this.mirrorEl) {
                    this.mirrorEl.style.display = 'none';
                }
                this.isVisible = bool;
            }
        };
        // always async
        ElementMirror.prototype.stop = function (needsRevertAnimation, callback) {
            var _this = this;
            var done = function () {
                _this.cleanup();
                callback();
            };
            if (needsRevertAnimation &&
                this.mirrorEl &&
                this.isVisible &&
                this.revertDuration && // if 0, transition won't work
                (this.deltaX || this.deltaY) // if same coords, transition won't work
            ) {
                this.doRevertAnimation(done, this.revertDuration);
            }
            else {
                setTimeout(done, 0);
            }
        };
        ElementMirror.prototype.doRevertAnimation = function (callback, revertDuration) {
            var mirrorEl = this.mirrorEl;
            var finalSourceElRect = this.sourceEl.getBoundingClientRect(); // because autoscrolling might have happened
            mirrorEl.style.transition =
                'top ' + revertDuration + 'ms,' +
                    'left ' + revertDuration + 'ms';
            common.applyStyle(mirrorEl, {
                left: finalSourceElRect.left,
                top: finalSourceElRect.top
            });
            common.whenTransitionDone(mirrorEl, function () {
                mirrorEl.style.transition = '';
                callback();
            });
        };
        ElementMirror.prototype.cleanup = function () {
            if (this.mirrorEl) {
                common.removeElement(this.mirrorEl);
                this.mirrorEl = null;
            }
            this.sourceEl = null;
        };
        ElementMirror.prototype.updateElPosition = function () {
            if (this.sourceEl && this.isVisible) {
                common.applyStyle(this.getMirrorEl(), {
                    left: this.sourceElRect.left + this.deltaX,
                    top: this.sourceElRect.top + this.deltaY
                });
            }
        };
        ElementMirror.prototype.getMirrorEl = function () {
            var sourceElRect = this.sourceElRect;
            var mirrorEl = this.mirrorEl;
            if (!mirrorEl) {
                mirrorEl = this.mirrorEl = this.sourceEl.cloneNode(true); // cloneChildren=true
                // we don't want long taps or any mouse interaction causing selection/menus.
                // would use preventSelection(), but that prevents selectstart, causing problems.
                mirrorEl.classList.add('fc-unselectable');
                mirrorEl.classList.add('fc-event-dragging');
                common.applyStyle(mirrorEl, {
                    position: 'fixed',
                    zIndex: this.zIndex,
                    visibility: '',
                    boxSizing: 'border-box',
                    width: sourceElRect.right - sourceElRect.left,
                    height: sourceElRect.bottom - sourceElRect.top,
                    right: 'auto',
                    bottom: 'auto',
                    margin: 0
                });
                this.parentNode.appendChild(mirrorEl);
            }
            return mirrorEl;
        };
        return ElementMirror;
    }());

    var WindowScrollGeomCache = /** @class */ (function (_super) {
        __extends(WindowScrollGeomCache, _super);
        function WindowScrollGeomCache(doesListening) {
            return _super.call(this, new common.WindowScrollController(), doesListening) || this;
        }
        WindowScrollGeomCache.prototype.getEventTarget = function () {
            return window;
        };
        WindowScrollGeomCache.prototype.computeClientRect = function () {
            return {
                left: this.scrollLeft,
                right: this.scrollLeft + this.clientWidth,
                top: this.scrollTop,
                bottom: this.scrollTop + this.clientHeight,
            };
        };
        // the window is the only scroll object that changes it's rectangle relative
        // to the document's topleft as it scrolls
        WindowScrollGeomCache.prototype.handleScrollChange = function () {
            this.clientRect = this.computeClientRect();
        };
        return WindowScrollGeomCache;
    }(ScrollGeomCache));

    // If available we are using native "performance" API instead of "Date"
    // Read more about it on MDN:
    // https://developer.mozilla.org/en-US/docs/Web/API/Performance
    var getTime = typeof performance === 'function' ? performance.now : Date.now;
    /*
    For a pointer interaction, automatically scrolls certain scroll containers when the pointer
    approaches the edge.

    The caller must call start + handleMove + stop.
    */
    var AutoScroller = /** @class */ (function () {
        function AutoScroller() {
            var _this = this;
            // options that can be set by caller
            this.isEnabled = true;
            this.scrollQuery = [window, '.fc-scroller'];
            this.edgeThreshold = 50; // pixels
            this.maxVelocity = 300; // pixels per second
            // internal state
            this.pointerScreenX = null;
            this.pointerScreenY = null;
            this.isAnimating = false;
            this.scrollCaches = null;
            // protect against the initial pointerdown being too close to an edge and starting the scroll
            this.everMovedUp = false;
            this.everMovedDown = false;
            this.everMovedLeft = false;
            this.everMovedRight = false;
            this.animate = function () {
                if (_this.isAnimating) { // wasn't cancelled between animation calls
                    var edge = _this.computeBestEdge(_this.pointerScreenX + window.pageXOffset, _this.pointerScreenY + window.pageYOffset);
                    if (edge) {
                        var now = getTime();
                        _this.handleSide(edge, (now - _this.msSinceRequest) / 1000);
                        _this.requestAnimation(now);
                    }
                    else {
                        _this.isAnimating = false; // will stop animation
                    }
                }
            };
        }
        AutoScroller.prototype.start = function (pageX, pageY, scrollStartEl) {
            if (this.isEnabled) {
                this.scrollCaches = this.buildCaches(scrollStartEl);
                this.pointerScreenX = null;
                this.pointerScreenY = null;
                this.everMovedUp = false;
                this.everMovedDown = false;
                this.everMovedLeft = false;
                this.everMovedRight = false;
                this.handleMove(pageX, pageY);
            }
        };
        AutoScroller.prototype.handleMove = function (pageX, pageY) {
            if (this.isEnabled) {
                var pointerScreenX = pageX - window.pageXOffset;
                var pointerScreenY = pageY - window.pageYOffset;
                var yDelta = this.pointerScreenY === null ? 0 : pointerScreenY - this.pointerScreenY;
                var xDelta = this.pointerScreenX === null ? 0 : pointerScreenX - this.pointerScreenX;
                if (yDelta < 0) {
                    this.everMovedUp = true;
                }
                else if (yDelta > 0) {
                    this.everMovedDown = true;
                }
                if (xDelta < 0) {
                    this.everMovedLeft = true;
                }
                else if (xDelta > 0) {
                    this.everMovedRight = true;
                }
                this.pointerScreenX = pointerScreenX;
                this.pointerScreenY = pointerScreenY;
                if (!this.isAnimating) {
                    this.isAnimating = true;
                    this.requestAnimation(getTime());
                }
            }
        };
        AutoScroller.prototype.stop = function () {
            if (this.isEnabled) {
                this.isAnimating = false; // will stop animation
                for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {
                    var scrollCache = _a[_i];
                    scrollCache.destroy();
                }
                this.scrollCaches = null;
            }
        };
        AutoScroller.prototype.requestAnimation = function (now) {
            this.msSinceRequest = now;
            requestAnimationFrame(this.animate);
        };
        AutoScroller.prototype.handleSide = function (edge, seconds) {
            var scrollCache = edge.scrollCache;
            var edgeThreshold = this.edgeThreshold;
            var invDistance = edgeThreshold - edge.distance;
            var velocity = // the closer to the edge, the faster we scroll
             ((invDistance * invDistance) / (edgeThreshold * edgeThreshold)) * // quadratic
                this.maxVelocity * seconds;
            var sign = 1;
            switch (edge.name) {
                case 'left':
                    sign = -1;
                // falls through
                case 'right':
                    scrollCache.setScrollLeft(scrollCache.getScrollLeft() + velocity * sign);
                    break;
                case 'top':
                    sign = -1;
                // falls through
                case 'bottom':
                    scrollCache.setScrollTop(scrollCache.getScrollTop() + velocity * sign);
                    break;
            }
        };
        // left/top are relative to document topleft
        AutoScroller.prototype.computeBestEdge = function (left, top) {
            var edgeThreshold = this.edgeThreshold;
            var bestSide = null;
            var scrollCaches = this.scrollCaches || [];
            for (var _i = 0, scrollCaches_1 = scrollCaches; _i < scrollCaches_1.length; _i++) {
                var scrollCache = scrollCaches_1[_i];
                var rect = scrollCache.clientRect;
                var leftDist = left - rect.left;
                var rightDist = rect.right - left;
                var topDist = top - rect.top;
                var bottomDist = rect.bottom - top;
                // completely within the rect?
                if (leftDist >= 0 && rightDist >= 0 && topDist >= 0 && bottomDist >= 0) {
                    if (topDist <= edgeThreshold && this.everMovedUp && scrollCache.canScrollUp() &&
                        (!bestSide || bestSide.distance > topDist)) {
                        bestSide = { scrollCache: scrollCache, name: 'top', distance: topDist };
                    }
                    if (bottomDist <= edgeThreshold && this.everMovedDown && scrollCache.canScrollDown() &&
                        (!bestSide || bestSide.distance > bottomDist)) {
                        bestSide = { scrollCache: scrollCache, name: 'bottom', distance: bottomDist };
                    }
                    if (leftDist <= edgeThreshold && this.everMovedLeft && scrollCache.canScrollLeft() &&
                        (!bestSide || bestSide.distance > leftDist)) {
                        bestSide = { scrollCache: scrollCache, name: 'left', distance: leftDist };
                    }
                    if (rightDist <= edgeThreshold && this.everMovedRight && scrollCache.canScrollRight() &&
                        (!bestSide || bestSide.distance > rightDist)) {
                        bestSide = { scrollCache: scrollCache, name: 'right', distance: rightDist };
                    }
                }
            }
            return bestSide;
        };
        AutoScroller.prototype.buildCaches = function (scrollStartEl) {
            return this.queryScrollEls(scrollStartEl).map(function (el) {
                if (el === window) {
                    return new WindowScrollGeomCache(false); // false = don't listen to user-generated scrolls
                }
                return new ElementScrollGeomCache(el, false); // false = don't listen to user-generated scrolls
            });
        };
        AutoScroller.prototype.queryScrollEls = function (scrollStartEl) {
            var els = [];
            for (var _i = 0, _a = this.scrollQuery; _i < _a.length; _i++) {
                var query = _a[_i];
                if (typeof query === 'object') {
                    els.push(query);
                }
                else {
                    els.push.apply(els, Array.prototype.slice.call(common.getElRoot(scrollStartEl).querySelectorAll(query)));
                }
            }
            return els;
        };
        return AutoScroller;
    }());

    var FeaturefulElementCopy = /** @class */ (function () {
        function FeaturefulElementCopy(containerEl, selector) {
            var _this = this;
            this.type = null;
            this.delay = null;
            this.minDistance = 0;
            this.touchScrollAllowed = true; // prevents drag from starting and blocks scrolling during drag
            this.mirrorNeedsRevert = false;
            this.isInteracting = false; // is the user validly moving the pointer? lasts until pointerup
            this.isDragging = false; // is it INTENTFULLY dragging? lasts until after revert animation
            this.isDelayEnded = false;
            this.isDistanceSurpassed = false;
            this.delayTimeoutId = null;
            this.onPointerCopy = function (ev) {
                _this.type = 'copy';
                _this.emitter.trigger('pointer-copy', ev);
                _this.onPointerDown(ev);
            };
            this.onPointerCut = function (ev) {
                _this.type = 'cut';
                _this.emitter.trigger('pointer-cut', ev);
                _this.onPointerDown(ev);
            };
            this.onPointerPaste = function (ev) {
                if (_this.type === null)
                    return;
                common.allowSelection(document.body);
                common.allowContextMenu(document.body);
                _this.emitter.trigger('pointer-paste', ev);
            };
            this.onPointerDown = function (ev) {
                common.preventSelection(document.body);
                common.preventContextMenu(document.body);
                _this.mirror.setIsVisible(true);
                _this.mirror.start(ev.subjectEl, ev.pageX, ev.pageY);
            };
            this.onMousedown = function (ev) {
                if (_this.type === null)
                    return;
                common.allowSelection(document.body);
                common.allowContextMenu(document.body);
                _this.emitter.trigger('pointer-paste', ev);
            };
            this.onMousemove = function (ev) {
                if (_this.type === null)
                    return;
                _this.emitter.trigger('mousemove', ev);
                // if (ev.origEvent.type !== 'scroll') {
                _this.mirror.handleMove(ev.pageX, ev.pageY);
                // }
            };
            this.cleanup = function () {
                _this.type = null;
                _this.mirror.stop(false, function () {
                });
                _this.emitter.trigger('cleanup', true);
            };
            this.containerEl = containerEl;
            this.emitter = new common.Emitter();
            this.mirror = new ElementMirror();
            this.autoScroller = new AutoScroller();
            var pointer = this.pointer = new PointerTracking(containerEl);
            pointer.emitter.on('pointer-copy', this.onPointerCopy);
            pointer.emitter.on('pointer-cut', this.onPointerCut);
            pointer.emitter.on('pointer-paste', this.onPointerPaste);
            pointer.emitter.on('mousedown', this.onMousedown);
            pointer.emitter.on('mousemove', this.onMousemove);
            pointer.emitter.on('cleanup', this.cleanup);
            if (selector) {
                pointer.selector = selector;
            }
        }
        FeaturefulElementCopy.prototype.destroy = function () {
            this.pointer.destroy();
        };
        FeaturefulElementCopy.prototype.setMirrorIsVisible = function (bool) {
            this.mirror.setIsVisible(bool);
        };
        return FeaturefulElementCopy;
    }());

    function buildDatePointApiWithContext(dateSpan, context) {
        var props = {};
        for (var _i = 0, _a = context.pluginHooks.datePointTransforms; _i < _a.length; _i++) {
            var transform = _a[_i];
            __assign(props, transform(dateSpan, context));
        }
        __assign(props, buildDatePointApi(dateSpan, context.dateEnv));
        return props;
    }
    function buildDatePointApi(span, dateEnv) {
        return {
            date: dateEnv.toDate(span.range.start),
            dateStr: dateEnv.formatIso(span.range.start, { omitTime: span.allDay }),
            allDay: span.allDay,
        };
    }

    var TYPE_EVENT;
    (function (TYPE_EVENT) {
        TYPE_EVENT[TYPE_EVENT["NONE"] = 0] = "NONE";
        TYPE_EVENT[TYPE_EVENT["COPY"] = 1] = "COPY";
        TYPE_EVENT[TYPE_EVENT["CUT"] = 2] = "CUT";
        TYPE_EVENT[TYPE_EVENT["PASTE"] = 3] = "PASTE";
        TYPE_EVENT[TYPE_EVENT["DUPLICATE"] = 4] = "DUPLICATE";
    })(TYPE_EVENT || (TYPE_EVENT = {}));
    var EventCopy = /** @class */ (function (_super) {
        __extends(EventCopy, _super);
        function EventCopy(settings) {
            var _this = _super.call(this, settings) || this;
            // internal state
            _this.subjectEl = null;
            _this.subjectSeg = null; // the seg being selected/dragged
            _this.eventRange = null;
            _this.relevantEvents = null; // the events being dragged
            _this.receivingContext = null;
            _this.validMutation = null;
            _this.mutatedRelevantEvents = null;
            _this.oldStyle = {};
            _this.handleCopy = function (ev) {
                _this.type = 'copy';
                _this.handleInputEvent(ev);
                _this.copyToClipboard();
            };
            _this.handleCut = function (ev) {
                _this.type = 'cut';
                _this.handleInputEvent(ev);
                _this.copyToClipboard();
            };
            _this.handleInputEvent = function (ev) {
                _this.clearDrag();
                var _a = _this, component = _a.component, manager = _a.manager;
                var initialContext = component.context;
                _this.subjectEl = ev.subjectEl;
                var subjectSeg = _this.subjectSeg = common.getElSeg(ev.subjectEl);
                var eventRange = _this.eventRange = subjectSeg.eventRange;
                var eventInstanceId = eventRange.instance.instanceId;
                _this.relevantEvents = common.getRelevantEvents(initialContext.getCurrentData().eventStore, eventInstanceId);
                var origTarget = ev.origEvent.target;
                var mirror = manager.mirror;
                var options = component.context.options;
                if (options.fixedMirrorParent) {
                    mirror.parentNode = options.fixedMirrorParent;
                }
                else {
                    mirror.parentNode = common.elementClosest(origTarget, '.fc');
                }
                mirror.revertDuration = options.dragRevertDuration;
            };
            _this.copyToClipboard = function () {
                var initialContext = _this.component.context;
                var eventDef = _this.eventRange.def;
                var eventInstance = _this.eventRange.instance;
                var eventApi = new common.EventApi(initialContext, eventDef, eventInstance);
                navigator.clipboard.writeText(JSON.stringify(eventApi.toJSON()));
            };
            _this.handleHitUpdate = function (hit, isFinal) {
                if (!_this.type || !hit)
                    return;
                var relevantEvents = _this.relevantEvents;
                var initialHit = _this.hitChecker.initialHit;
                // states based on new hit
                var receivingContext = null;
                var mutation = null;
                var mutatedRelevantEvents = null;
                var isInvalid = false;
                var interaction = {
                    affectedEvents: relevantEvents,
                    mutatedEvents: common.createEmptyEventStore(),
                    isEvent: true
                };
                receivingContext = hit.context;
                mutation = computeEventMutation(initialHit, hit, receivingContext.getCurrentData().pluginHooks.eventDragMutationMassagers);
                if (mutation && relevantEvents) {
                    mutatedRelevantEvents = common.applyMutationToEventStore(relevantEvents, receivingContext.getCurrentData().eventUiBases, mutation, receivingContext);
                    interaction.mutatedEvents = mutatedRelevantEvents;
                    if (!common.isInteractionValid(interaction, hit.dateProfile, receivingContext)) {
                        isInvalid = true;
                        mutation = null;
                        mutatedRelevantEvents = null;
                        interaction.mutatedEvents = common.createEmptyEventStore();
                    }
                }
                _this.displayDrag(receivingContext, interaction);
                if (!isInvalid) {
                    common.enableCursor();
                }
                else {
                    common.disableCursor();
                }
                _this.manager.setMirrorIsVisible(!hit || !common.getElRoot(_this.subjectEl).querySelector('.fc-event-mirror'));
                if (_this.type === 'copy') {
                    var parentElement = _this.subjectEl.parentElement;
                    parentElement.style.visibility = '';
                    parentElement.style.opacity = '0.75';
                    _this.oldStyle.zIndex === void 0 && (_this.oldStyle.zIndex = parentElement.style.zIndex);
                    parentElement.style.zIndex = String(parentElement.parentElement.childElementCount + 1);
                }
                _this.receivingContext = receivingContext;
                _this.validMutation = mutation;
                _this.mutatedRelevantEvents = mutatedRelevantEvents;
            };
            _this.handlePaste = function (ev) {
                if (!_this.type)
                    return;
                _this.handleHitUpdate(_this.hitChecker.finalHit, true);
                var component = _this.component;
                var options = component.context.options;
                // @ts-ignore
                var initialContext = _this.component.context;
                var initialView = initialContext.viewApi;
                var _a = _this, receivingContext = _a.receivingContext, validMutation = _a.validMutation;
                var eventDef = _this.eventRange.def;
                var eventInstance = _this.eventRange.instance;
                var eventApi = new common.EventApi(initialContext, eventDef, eventInstance);
                var relevantEvents = _this.relevantEvents;
                var mutatedRelevantEvents = _this.mutatedRelevantEvents;
                var finalHit = _this.hitChecker.finalHit;
                if (validMutation) {
                    if (receivingContext === initialContext) {
                        var newEventApi = new common.EventApi(initialContext, mutatedRelevantEvents.defs[eventDef.defId], eventInstance ? mutatedRelevantEvents.instances[eventInstance.instanceId] : null);
                        if (options.previewCopy === void 0 || options.previewCopy === true) {
                            if (_this.type === 'copy') {
                                var newEvent = _this.cloneEvent();
                                initialContext.calendarApi.addEvent(newEvent);
                            }
                            else if (_this.type === 'cut') {
                                initialContext.dispatch({
                                    type: 'MERGE_EVENTS',
                                    eventStore: mutatedRelevantEvents
                                });
                            }
                        }
                        var eventChangeArg = {
                            oldEvent: eventApi,
                            event: newEventApi,
                            relatedEvents: common.buildEventApis(mutatedRelevantEvents, initialContext, eventInstance),
                            revert: function () {
                                initialContext.dispatch({
                                    type: 'MERGE_EVENTS',
                                    eventStore: relevantEvents // the pre-change data
                                });
                            }
                        };
                        var transformed = {};
                        for (var _i = 0, _b = initialContext.getCurrentData().pluginHooks.eventDropTransformers; _i < _b.length; _i++) {
                            var transformer = _b[_i];
                            __assign(transformed, transformer(validMutation, initialContext));
                        }
                        // @ts-ignore
                        initialContext.emitter.trigger('eventCopy', __assign(__assign(__assign({}, eventChangeArg), transformed), { type: _this.type, el: ev.subjectEl, delta: validMutation.datesDelta, jsEvent: ev.origEvent, view: initialView }));
                    }
                    else if (receivingContext) {
                        if (options.previewCopy === void 0 || options.previewCopy === true) {
                            if (_this.type === 'cut') {
                                var eventRemoveArg = {
                                    event: eventApi,
                                    relatedEvents: common.buildEventApis(relevantEvents, initialContext, eventInstance),
                                    revert: function () {
                                        initialContext.dispatch({
                                            type: 'MERGE_EVENTS',
                                            eventStore: relevantEvents
                                        });
                                    }
                                };
                                initialContext.dispatch({
                                    type: 'REMOVE_EVENTS',
                                    eventStore: relevantEvents
                                });
                                initialContext.emitter.trigger('eventRemove', eventRemoveArg);
                            }
                            var newEvent = _this.cloneEvent();
                            receivingContext.calendarApi.addEvent(newEvent);
                        }
                        var addedEventDef = mutatedRelevantEvents.defs[eventDef.defId];
                        var addedEventInstance = mutatedRelevantEvents.instances[eventInstance.instanceId];
                        var addedEventApi = new common.EventApi(receivingContext, addedEventDef, addedEventInstance);
                        // @ts-ignore
                        receivingContext.emitter.trigger('eventCopy', __assign(__assign({}, buildDatePointApiWithContext(finalHit.dateSpan, receivingContext)), { draggedEl: ev.subjectEl, jsEvent: ev.origEvent, type: _this.type, event: addedEventApi, oldEvent: eventApi, relatedEvents: common.buildEventApis(mutatedRelevantEvents, receivingContext, addedEventInstance), relatedOldEvents: common.buildEventApis(relevantEvents, initialContext, eventInstance), view: finalHit.context.viewApi }));
                    }
                }
                _this.cleanup();
            };
            _this.cleanup = function () {
                _this.clearDrag();
                if (_this.subjectEl !== null) {
                    var parentElement = _this.subjectEl.parentElement;
                    parentElement.style.visibility = '';
                    parentElement.style.opacity = '';
                    parentElement.style.zIndex = _this.oldStyle.zIndex;
                    _this.oldStyle = {};
                }
                _this.type = null;
                _this.subjectSeg = null;
                _this.eventRange = null;
                _this.relevantEvents = null;
                _this.receivingContext = null;
                _this.validMutation = null;
                _this.mutatedRelevantEvents = null;
            };
            // @ts-ignore
            _this.manager = new FeaturefulElementCopy(settings.el);
            _this.manager.pointer.selector = EventCopy.SELECTOR;
            var hitChecker = _this.hitChecker = new HitChecker(_this.manager, common.interactionSettingsStore);
            hitChecker.useSubjectCenter = settings.useEventCenter;
            hitChecker.emitter.on('pointer-copy', _this.handleCopy);
            hitChecker.emitter.on('pointer-cut', _this.handleCut);
            hitChecker.emitter.on('hitupdate', _this.handleHitUpdate);
            hitChecker.emitter.on('paste', _this.handlePaste);
            hitChecker.emitter.on('cleanup', _this.cleanup);
            return _this;
        }
        EventCopy.prototype.destroy = function () {
            this.manager.destroy();
        };
        EventCopy.prototype.displayDrag = function (nextContext, state) {
            var initialContext = this.component.context;
            var prevContext = this.receivingContext;
            // does the previous calendar need to be cleared?
            if (prevContext && prevContext !== nextContext) {
                // does the initial calendar need to be cleared?
                // if so, don't clear all the way. we still need to to hide the affectedEvents
                if (prevContext === initialContext) {
                    prevContext.dispatch({
                        type: 'SET_EVENT_DRAG',
                        state: {
                            affectedEvents: state.affectedEvents,
                            mutatedEvents: common.createEmptyEventStore(),
                            isEvent: true
                        }
                    });
                    // completely clear the old calendar if it wasn't the initial
                }
                else {
                    prevContext.dispatch({ type: 'UNSET_EVENT_DRAG' });
                }
            }
            if (nextContext) {
                nextContext.dispatch({ type: 'SET_EVENT_DRAG', state: state });
            }
        };
        EventCopy.prototype.clearDrag = function () {
            var initialCalendar = this.component.context;
            var receivingContext = this.receivingContext;
            if (receivingContext) {
                receivingContext.dispatch({ type: 'UNSET_EVENT_DRAG' });
            }
            // the initial calendar might have an dummy drag state from displayDrag
            if (initialCalendar !== receivingContext) {
                initialCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });
            }
        };
        EventCopy.prototype.cloneEvent = function () {
            var initialContext = this.component.context;
            var eventDef = this.eventRange.def;
            var eventInstance = this.eventRange.instance;
            var newEventApi = new common.EventApi(initialContext, this.mutatedRelevantEvents.defs[eventDef.defId], eventInstance ? this.mutatedRelevantEvents.instances[eventInstance.instanceId] : null);
            var finalHit = this.hitChecker.finalHit;
            var resourceId = finalHit.dateSpan.resourceId;
            var newEvent = __assign(__assign({}, this.eventRange.ui), this.eventRange.def);
            // @ts-ignore
            newEvent.resourceId = resourceId;
            delete newEvent.ui;
            delete newEvent.defId;
            delete newEvent.publicId;
            delete newEvent.resourceIds;
            delete newEvent.extendedProps;
            delete newEvent.recurringDef;
            delete newEvent.sourceId;
            newEvent.allDay = newEventApi.allDay;
            var offset = new Date().getTimezoneOffset() * 60 * 1000;
            // @ts-ignore
            newEvent.start = new Date(this.mutatedRelevantEvents.instances[eventInstance.instanceId].range.start.getTime() + offset);
            // @ts-ignore
            newEvent.end = new Date(this.mutatedRelevantEvents.instances[eventInstance.instanceId].range.end.getTime() + offset);
            return newEvent;
        };
        EventCopy.SELECTOR = '.fc-event';
        return EventCopy;
    }(common.Interaction));
    function computeEventMutation(hit0, hit1, massagers) {
        var dateSpan0 = hit0.dateSpan;
        var dateSpan1 = hit1.dateSpan;
        var date0 = dateSpan0.range.start;
        var date1 = dateSpan1.range.start;
        var standardProps = {};
        if (dateSpan0.allDay !== dateSpan1.allDay) {
            standardProps.allDay = dateSpan1.allDay;
            standardProps.hasEnd = hit1.context.options.allDayMaintainDuration;
            if (dateSpan1.allDay) {
                date0 = common.startOfDay(date0);
            }
        }
        var delta = common.diffDates(date0, date1, hit0.context.dateEnv, hit0.componentId === hit1.componentId ?
            hit0.largeUnit :
            null);
        if (delta.milliseconds) { // has hours/minutes/seconds
            standardProps.allDay = false;
        }
        var mutation = {
            datesDelta: delta,
            standardProps: standardProps
        };
        for (var _i = 0, massagers_1 = massagers; _i < massagers_1.length; _i++) {
            var massager = massagers_1[_i];
            massager(mutation, hit0, hit1);
        }
        return mutation;
    }

    // import { PointerTrigger } from './PointerTrigger'
    // import { PointerDragging } from '../dnd/PointerDragging'
    // import { EventDragging } from './EventDragging'
    var CopyPasteInteract = /** @class */ (function () {
        function CopyPasteInteract(context) {
            var _this = this;
            this.isRecentPointerDateSelect = false; // wish we could use a selector to detect date selection, but uses hit system
            this.matchesCancel = false;
            this.matchesEvent = false;
            this.onSelect = function (selectInfo) {
                if (selectInfo.jsEvent) {
                    _this.isRecentPointerDateSelect = true;
                }
            };
            this.onDocumentPointerDown = function (pev) {
                var unselectCancel = _this.context.options.unselectCancel;
                var downEl = common.getEventTargetViaRoot(pev.origEvent);
                _this.matchesCancel = !!common.elementClosest(downEl, unselectCancel);
                _this.matchesEvent = !!common.elementClosest(downEl, EventCopy.SELECTOR); // interaction started on an event?
            };
            this.onDocumentPointerUp = function (pev) {
                // let { context } = this
                // let { documentPointer } = this
                // let calendarState = context.getCurrentData()
                // touch-scrolling should never unfocus any type of selection
                // if (!documentPointer.wasTouchScroll) {
                //   if (
                //     calendarState.dateSelection && // an existing date selection?
                //     !this.isRecentPointerDateSelect // a new pointer-initiated date selection since last onDocumentPointerUp?
                //   ) {
                //     let unselectAuto = context.options.unselectAuto
                //
                //     if (unselectAuto && (!unselectAuto || !this.matchesCancel)) {
                //       context.calendarApi.unselect(pev)
                //     }
                //   }
                //
                //   if (
                //     calendarState.eventSelection && // an existing event selected?
                //     !this.matchesEvent // interaction DIDN'T start on an event
                //   ) {
                //     context.dispatch({ type: 'UNSELECT_EVENT' })
                //   }
                // }
                _this.isRecentPointerDateSelect = false;
            };
            this.context = context;
            // let documentPointer = this.documentPointer = new PointerTracking(document)
            // documentPointer.shouldIgnoreMove = true
            // documentPointer.shouldWatchScroll = false
        }
        CopyPasteInteract.prototype.destroy = function () {
            // this.context.emitter.off('select', this.onSelect)
            // this.documentPointer.destroy()
        };
        return CopyPasteInteract;
    }());

    var OPTION_REFINERS = {
        previewCopy: Boolean,
    };
    var LISTENER_REFINERS = {
        eventCopy: common.identity,
    };

    var plugin = common.createPlugin({
        componentInteractions: [EventCopy],
        calendarInteractions: [CopyPasteInteract],
        optionRefiners: OPTION_REFINERS,
        listenerRefiners: LISTENER_REFINERS,
    });

    common.globalPlugins.push(plugin);

    exports.default = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

}({}, FullCalendar));
